import { Base, Type } from "igniteui-angular-core";
import { QuartileCalculation } from "./QuartileCalculation";
/**
 * Exposes properties related to the <i>BoxAndWhisker</i> chart type.
 * <p class="body">
 * This class is immutable.
 * </p>
 * @see [[Series.boxAndWhiskerSettings]]
 */
export declare class BoxAndWhiskerSettings extends Base {
    static $t: Type;
    /**
     * @hidden
     */
    static readonly _c: boolean;
    /**
     * @hidden
     */
    static readonly _f: boolean;
    /**
     * @hidden
     */
    static readonly _e: boolean;
    /**
     * @hidden
     */
    static readonly _d: boolean;
    /**
     * @hidden
     */
    static readonly _a: QuartileCalculation;
    constructor(showInnerPoints: boolean, showOutlierPoints: boolean, showMeanMarkers: boolean, showMeanLine: boolean, quartileCalculation: QuartileCalculation);
    private _showInnerPoints;
    /**
     * Determines whether inner points appear on a box and whisker chart.
     * <p class="body">
     * Inner points appear between the lower whisker line and the upper whisker line.
     * </p>
     * @see [[showOutlierPoints]]
     */
    get showInnerPoints(): boolean;
    set showInnerPoints(a: boolean);
    private _showOutlierPoints;
    /**
     * Determines whether outlier points appear on a box and whisker chart.
     * <p class="body">
     * Outlier points appear either below the lower whisker line or above the upper whisker line.
     * </p>
     * @see [[showInnerPoints]]
     */
    get showOutlierPoints(): boolean;
    set showOutlierPoints(a: boolean);
    private _showMeanMarkers;
    /**
     * Determines whether mean markers appear on a box and whisker chart.
     * @see [[showMeanLine]]
     */
    get showMeanMarkers(): boolean;
    set showMeanMarkers(a: boolean);
    private _showMeanLine;
    /**
     * Determines whether a mean line appears on a box and whisker chart.
     * <p class="body">
     * Mean lines connect the means of the boxes.
     * </p>
     * @see [[showMeanMarkers]]
     */
    get showMeanLine(): boolean;
    set showMeanLine(a: boolean);
    private _quartileCalculation;
    /**
     * Determines the method for median calculation.
     */
    get quartileCalculation(): QuartileCalculation;
    set quartileCalculation(a: QuartileCalculation);
}
