import { Type } from "igniteui-angular-core";
/**
 * Represents the various calendar types available for the [[FixedValuesFilter]]
 * @see [[FixedValuesFilter.calendarType]]
 */
export declare enum CalendarType {
    /**
     * The Gregorian calendar should be used.
     */
    Gregorian = 0,
    /**
     * The Arabic version of the Gregorian calendar should be used.
     */
    GregorianArabic = 1,
    /**
     * The Middle East French version of the Gregorian calendar should be used.
     */
    GregorianMeFrench = 2,
    /**
     * The US English version of the Gregorian calendar should be used.
     */
    GregorianUs = 3,
    /**
     * The transliterated English version of the Gregorian calendar should be used.
     */
    GregorianXlitEnglish = 4,
    /**
     * The transliterated French version of the Gregorian calendar should be used.
     */
    GregorianXlitFrench = 5,
    /**
     * The Hebrew lunar calendar, as described by the Gauss formula for Passover and The Complete Restatement of Oral Law,
     * should be used.
     */
    Hebrew = 6,
    /**
     * The Hijri lunar calendar, as described by the Kingdom of Saudi Arabia, Ministry of Islamic Affairs, Endowments, Da‘wah
     * and Guidance, should be used.
     */
    Hijri = 7,
    /**
     * The Japanese Emperor Era calendar, as described by Japanese Industrial Standard JIS X 0301, should be used.
     */
    Japan = 8,
    /**
     * The Korean Tangun Era calendar, as described by Korean Law Enactment No. 4, should be used.
     */
    Korea = 9,
    /**
     * Specifies that no calendar should be used.
     */
    None = 10,
    /**
     * The Saka Era calendar, as described by the Calendar Reform Committee of India, as part of the Indian Ephemeris and Nautical
     * Almanac, should be used.
     */
    Saka = 11,
    /**
     * The Taiwanese calendar, as defined by the Chinese National Standard CNS 7648, should be used.
     */
    Taiwan = 12,
    /**
     * The Thai calendar, as defined by the Royal Decree of H.M. King Vajiravudh (Rama VI) in Royal Gazette B. E. 2456 (1913 A.D.)
     * and by the decree of Prime Minister Phibunsongkhram (1941 A.D.) to start the year on the Gregorian January 1 and to map year
     * zero to Gregorian year 543 B.C., should be used.
     */
    Thai = 13
}
/**
 * @hidden
 */
export declare let CalendarType_$type: Type;
