import { ChartFillBase } from "./ChartFillBase";
import { GradientType } from "./GradientType";
import { IEnumerable$1, Type } from "igniteui-angular-core";
import { GradientStop } from "./GradientStop";
/**
 * Represents a gradient fill for a chart element.
 * @see [[GradientStop]]
 * @see [[GradientType]]
 */
export declare class ChartGradientFill extends ChartFillBase {
    static $t: Type;
    private _z;
    private _w;
    private _aa;
    constructor(gradientType: GradientType, stops: IEnumerable$1<GradientStop>, angle?: number);
    constructor(..._rest: any[]);
    /**
     * Returns the angle defining the path along which the gradient extends;
     * applicable only for linear gradients.
     * <p class="body">
     * This property only applies to linear gradients.
     * </p>
     * <p class="body">
     * This property defines the angle the gradient path makes with the horizontal plane.
     * </p>
     * <p class="body">
     * For example, a gradient which extends from left to right can be changed to extend from
     * bottom to top by setting this property to 90 degrees.
     * </p>
     * <p class="body">
     * A value of 45 degrees yields a gradient which extends from the bottom left corner
     * of the containing element to the top right corner.
     * </p>
     */
    get angle(): number;
    /**
     * Defines the style for the gradient.
     */
    get gradientType(): GradientType;
    /**
     * Returns an enumerable list containing each [[GradientStop]]
     * associated with this gradient, ordered by their [[GradientStop.position]].
     * @return An array of [[GradientStop]] instances.
     */
    getStops(): IEnumerable$1<GradientStop>;
    /**
     * @hidden
     */
    private _ac;
}
