import { Type } from "igniteui-angular-core";
/**
 * Determines what types of cell values are allowed and how the cell value is validated against the constraint(s).
 * @see [[ValueConstraintDataValidationRule.validationCriteria]]
 * @see [[OneConstraintDataValidationRule]]
 * @see [[TwoConstraintDataValidationRule]]
 */
export declare enum DataValidationCriteria {
    /**
     * Only numbers are allowed and they cannot have a fractional part.
     * When comparing against constraints, the cell value is used.
     */
    WholeNumber = 1,
    /**
     * Only numbers are allowed.
     * When comparing against constraints, the cell value is used.
     */
    Decimal = 2,
    /**
     * Dates with or without time portions are allowed as well as numbers equivalent to valid dates.
     * When comparing against constraints, the cell value is used.
     */
    Date = 4,
    /**
     * Times are allowed are well as numbers equivalent to valid times without a date portion.
     * When comparing against constraints, the cell value is used.
     */
    Time = 5,
    /**
     * Non-error values are allowed.
     * When comparing against constraints, the length of the cell value's text equivalent is used.
     */
    TextLength = 6
}
/**
 * @hidden
 */
export declare let DataValidationCriteria_$type: Type;
