import { Type } from "igniteui-angular-core";
/**
 * Constants which determine whether a chart element is positioned
 * automatically or at a specific location in the chart space.
 */
export declare enum ElementPosition {
    /**
     * The chart element is positioned automatically as appropriate
     * for that element. For example, a [[ChartTitle]] is typically
     * positioned at the top of the chart space, centered horizontally.
     */
    Automatic = -4105,
    /**
     * Custom.
     */
    Custom = -4114,
    /**
     * The chart element is positioned above the plot area, centered horizontally.
     * Applicable only for <i>BoxAndWhisker</i>, <i>Histogram</i>, <i>Pareto</i>, <i>Sunburst</i>, <i>Treemap</i>, and <i>Waterfall</i> chart types.
     */
    Top = 100,
    /**
     * The chart element is positioned to the left of the plot area, centered vertically.
     * Applicable only for <i>BoxAndWhisker</i>, <i>Histogram</i>, <i>Pareto</i>, <i>Sunburst</i>, <i>Treemap</i>, and <i>Waterfall</i> chart types.
     */
    Left = 101,
    /**
     * The chart element is positioned to the right of the plot area, centered vertically.
     * Applicable only for <i>BoxAndWhisker</i>, <i>Histogram</i>, <i>Pareto</i>, <i>Sunburst</i>, <i>Treemap</i>, and <i>Waterfall</i> chart types.
     */
    Right = 102,
    /**
     * The chart element is positioned under the plot area, centered horizontally.
     * Applicable only for <i>BoxAndWhisker</i>, <i>Histogram</i>, <i>Pareto</i>, <i>Sunburst</i>, <i>Treemap</i>, and <i>Waterfall</i> chart types.
     */
    Bottom = 103,
    /**
     * The chart element is positioned above the plot area, horizontally aligned with the left edge.
     * Applicable only for <i>BoxAndWhisker</i>, <i>Histogram</i>, <i>Pareto</i>, <i>Sunburst</i>, <i>Treemap</i>, and <i>Waterfall</i> chart types.
     */
    TopLeft = 104,
    /**
     * The chart element is positioned above the plot area, horizontally aligned with the right edge.
     * Applicable only for <i>BoxAndWhisker</i>, <i>Histogram</i>, <i>Pareto</i>, <i>Sunburst</i>, <i>Treemap</i>, and <i>Waterfall</i> chart types.
     */
    TopRight = 105,
    /**
     * The chart element is positioned below the plot area, horizontally aligned with the left edge.
     * Applicable only for <i>BoxAndWhisker</i>, <i>Histogram</i>, <i>Pareto</i>, <i>Sunburst</i>, <i>Treemap</i>, and <i>Waterfall</i> chart types.
     */
    BottomLeft = 106,
    /**
     * The chart element is positioned below the plot area, horizontally aligned with the right edge.
     * Applicable only for <i>BoxAndWhisker</i>, <i>Histogram</i>, <i>Pareto</i>, <i>Sunburst</i>, <i>Treemap</i>, and <i>Waterfall</i> chart types.
     */
    BottomRight = 107,
    /**
     * The chart element is positioned to the left of the plot area, vertically aligned with the top edge.
     * Applicable only for <i>BoxAndWhisker</i>, <i>Histogram</i>, <i>Pareto</i>, <i>Sunburst</i>, <i>Treemap</i>, and <i>Waterfall</i> chart types.
     */
    LeftTop = 108,
    /**
     * The chart element is positioned to the left of the plot area, vertically aligned with the bottom edge.
     * Applicable only for <i>BoxAndWhisker</i>, <i>Histogram</i>, <i>Pareto</i>, <i>Sunburst</i>, <i>Treemap</i>, and <i>Waterfall</i> chart types.
     */
    LeftBottom = 109,
    /**
     * The chart element is positioned to the right of the plot area, vertically aligned with the top edge.
     * Applicable only for <i>BoxAndWhisker</i>, <i>Histogram</i>, <i>Pareto</i>, <i>Sunburst</i>, <i>Treemap</i>, and <i>Waterfall</i> chart types.
     */
    RightTop = 110,
    /**
     * The chart element is positioned to the right of the plot area, vertically aligned with the bottom edge.
     * Applicable only for <i>BoxAndWhisker</i>, <i>Histogram</i>, <i>Pareto</i>, <i>Sunburst</i>, <i>Treemap</i>, and <i>Waterfall</i> chart types.
     */
    RightBottom = 111
}
/**
 * @hidden
 */
export declare let ElementPosition_$type: Type;
