import { Type } from "igniteui-angular-core";
/**
 * Constants which specify the manner in which a cell value is
 * compared against one or more operands.
 */
export declare enum FormatConditionOperator {
    /**
     * The condition evaluates to true if the cell value is between
     * the values of the two operands.
     */
    Between = 0,
    /**
     * The condition evaluates to true if the cell value is not between
     * the values of the two operands.
     */
    NotBetween = 1,
    /**
     * The condition evaluates to true if the cell value is equal to
     * the value of the operand.
     */
    Equal = 2,
    /**
     * The condition evaluates to true if the cell value is not equal to
     * the value of the operand.
     */
    NotEqual = 3,
    /**
     * The condition evaluates to true if the cell value is greater than
     * the value of the operand.
     */
    Greater = 4,
    /**
     * The condition evaluates to true if the cell value is less than
     * the value of the operand.
     */
    Less = 5,
    /**
     * The condition evaluates to true if the cell value is greater than
     * or equal to the value of the operand.
     */
    GreaterEqual = 6,
    /**
     * The condition evaluates to true if the cell value is less than
     * or equal to the value of the operand.
     */
    LessEqual = 7
}
/**
 * @hidden
 */
export declare let FormatConditionOperator_$type: Type;
