import { Type } from "igniteui-angular-core";
/**
 * Constants which define the manner in which a cell's date/time value is compared
 * when evaluating a [[DateTimeConditionalFormat]].
 */
export declare enum FormatConditionTimePeriod {
    /**
     * The cell's date/time value is compared to the current date.
     */
    Today = 0,
    /**
     * The cell's date/time value is compared to the day after the current date.
     */
    Tomorrow = 1,
    /**
     * The cell's date/time value is compared to the day before the current date.
     */
    Yesterday = 2,
    /**
     * The cell's date/time value is compared to the current week, as defined by the
     * [[CalendarWeekRule]].
     */
    ThisWeek = 3,
    /**
     * The cell's date/time value is compared to the week after the current one, as defined by the
     * [[CalendarWeekRule]].
     */
    NextWeek = 4,
    /**
     * The cell's date/time value is compared to the week before the current one, as defined by the
     * [[CalendarWeekRule]].
     */
    LastWeek = 5,
    /**
     * The cell's date/time value meets the condition if it lies within the range
     * of the past seven days, without regard to the
     * [[CalendarWeekRule]].
     */
    LastSevenDays = 6,
    /**
     * The cell's date/time value is compared to the current month.
     */
    ThisMonth = 7,
    /**
     * The cell's date/time value is compared to the month after the current one.
     */
    NextMonth = 8,
    /**
     * The cell's date/time value is compared to the month before the current one.
     */
    LastMonth = 9
}
/**
 * @hidden
 */
export declare let FormatConditionTimePeriod_$type: Type;
