import { Base, Type } from "igniteui-angular-core";
import { WorkbookColorInfo } from "./WorkbookColorInfo";
/**
 * Describes the location and color of a transition
 * point in a <see cref="ChartGradientFill">gradient</see>.
 */
export declare class GradientStop extends Base {
    static $t: Type;
    private _a;
    private _c;
    constructor(color: WorkbookColorInfo, position: number);
    /**
     * Returns the color for this gradient stop.
     */
    get color(): WorkbookColorInfo;
    /**
     * Returns the position of this gradient stop, as a number between 0 and 1.
     * <p class="body">
     * The value assigned to the Position property represents a fraction
     * of the area to which the gradient is applied.
     * </p>
     * <p class="body">
     * For example, to add two stops of equal length, create one GradientStop
     * with a position of .5 and another with a position of 1.0.
     * </p>
     * <p class="body">
     * The first stop extends from the origin to the halfway point of the fill area,
     * and the second extends from the end of the first to the endpoint of the area
     * to be filled. For three uniform stops, use .33. .67, and 1.0 as the respective
     * positions; for four, use .25, .5, .75, and 1.0, etc.
     * </p>
     */
    get position(): number;
    /**
     * @hidden
     */
    static _e(a: number, b?: string): void;
}
