import { Type } from "igniteui-angular-core";
/**
 * Represents the various operators which can be used when validating the cell value against a constraint.
 * @see [[OneConstraintDataValidationRule]]
 * @see [[OneConstraintDataValidationRule.setConstraint]]
 * @see [[OneConstraintDataValidationRule.setConstraint]]
 * @see [[OneConstraintDataValidationRule.getConstraintFormula]]
 * @see [[OneConstraintDataValidationRule.getConstraintFormula]]
 * @see [[OneConstraintDataValidationRule.setConstraintFormula]]
 * @see [[OneConstraintDataValidationRule.setConstraintFormula]]
 * @see [[ValueConstraintDataValidationRule.validationCriteria]]
 */
export declare enum OneConstraintDataValidationOperator {
    /**
     * Only allows the cell value if it or its text length, depending on the validation criteria, is equal to the
     * constraint applied to the validation rule.
     */
    EqualTo = 2,
    /**
     * Only allows the cell value if it or its text length, depending on the validation criteria, is not equal to the
     * constraint applied to the validation rule.
     */
    NotEqualTo = 3,
    /**
     * Only allows the cell value if it or its text length, depending on the validation criteria, is greater than the
     * constraint applied to the validation rule.
     */
    GreaterThan = 4,
    /**
     * Only allows the cell value if it or its text length, depending on the validation criteria, is greater than or equal to the
     * constraint applied to the validation rule.
     */
    GreaterThanOrEqualTo = 6,
    /**
     * Only allows the cell value if it or its text length, depending on the validation criteria, is less than the
     * constraint applied to the validation rule.
     */
    LessThan = 5,
    /**
     * Only allows the cell value if it or its text length, depending on the validation criteria, is less than or equal to the
     * constraint applied to the validation rule.
     */
    LessThanOrEqualTo = 7
}
/**
 * @hidden
 */
export declare let OneConstraintDataValidationOperator_$type: Type;
