import { Type } from "igniteui-angular-core";
/**
 * Represents the various units in which a column width can be represented.
 * @see [[Worksheet.getDefaultColumnWidth]]
 * @see [[Worksheet.setDefaultColumnWidth]]
 * @see [[WorksheetColumn.getWidth]]
 * @see [[WorksheetColumn.setWidth]]
 */
export declare enum WorksheetColumnWidthUnit {
    /**
     * The column width is represented in units of the '0' digit character width, including column padding. The digit is
     * measured with the default font for the workbook. The padding is a few pixels on either side of the column plus an
     * additional pixel for the gridline.
     */
    Character = 0,
    /**
     * The column width is represented in 256ths of the '0' digit character width, including column padding, which means this
     * value will be 256 times the width expressed in Character units. The digit is measured with the default font for the
     * workbook. The padding is a few pixels on either side of the column plus an additional pixel for the gridline. These
     * units are the units in which the [[WorksheetColumn.width]] and [[Worksheet.defaultColumnWidth]]
     * properties are expressed.
     */
    Character256th = 1,
    /**
     * The column width is represented in units of the '0' digit character width, excluding padding. The digit is measured with
     * the default font for the workbook. These units are the units in which Microsoft Excel displays column widths to the user
     * and accepts new column widths from the user in the 'Column Width' dialog.
     */
    CharacterPaddingExcluded = 2,
    /**
     * The column width is represented in pixels.
     */
    Pixel = 3,
    /**
     * The column width is represented in points.
     */
    Point = 4,
    /**
     * The column width is represented in twips (20ths of a point).
     */
    Twip = 5
}
/**
 * @hidden
 */
export declare let WorksheetColumnWidthUnit_$type: Type;
